
/*******************************************************************************

  GPS Com PIC16F628A

  Display LCD 16x2

  Autor: Cludio Lrios

  Data: 18/09/2019

  Objetivo: Mostrar em um LCD 16x2 a localizao atual  dispositivo
  usando mdulo GPS NEO-6M.

  Valor mostrado est em graus/minutos/segundos decimais
  Padro do Googlemaps : graus/minutos decimais
  Para converter valor mostrado para padro Googlemaps multiplique os
  minutos por 100 e divida por 60. O restante deve ser somado com os segundos
  decimais.
  Ex. 2332.49444 (mostrado no LCD)

  Pegue os graus e multiplique por 100 e divida por 60:
  32x100/60= 53.33333

  Obtido foi 53 graus decimais com resto de .33333

  Somando resto (.33333) com os segundos decimais (.49444):

  49444+33333=82777

  Ento valor convertido ser:
  2353.82777

  Compilado com CCS C compiler (Version 3.200)
  Arquivo para fins didticos apenas.
  Este arquivo  parte integrante do blog PicSource

*******************************************************************************/

#include <16F628A.h>
#use delay(clock=4000000)//xtal de 4mhz externo nos pinos 15 e 16
#use rs232(baud=9600,xmit=pin_b2, rcv=pin_b1)


#fuses NOWDT,XT, PUT, NOPROTECT, BROWNOUT, MCLR, NOLVP, NOCPD
#zero_ram//zera no nicio
#use fast_io(a)//modo rpido
#use fast_io(b)//modo rpido
//bytes
#byte trisb = 0x86
#byte trisa = 0x85
#byte tmr0  = 0x01
#byte portb = 0x06
#byte porta = 0x05
#byte rcreg = 0x1A

// lcd - escolhas dos pinos para envio ao LCD
//obs. no use 6.1 e nem 6.2 pois esto sendo usados pela serial.

#bit   en     = 0x06.0 //rb0  lcd 'enable' pino 6
#bit   rs     = 0x06.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   db4    = 0x06.4 //rb4  db4 pino 10
#bit   db5    = 0x06.5 //rb5  db5 pino 11
#bit   db6    = 0x06.6 //rb6  db6 pino 12
#bit   db7    = 0x06.7 //rb7  db7 pino 13
#bit   tris_en     = 0x86.2 //rb2  lcd 'enable' pino 8
#bit   tris_rs     = 0x86.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   tris_db4    = 0x86.4 //rb4  dir db4 pino 10
#bit   tris_db5    = 0x86.5 //rb5  dir db5 pino 11
#bit   tris_db6    = 0x86.6 //rb6  dir db6 pino 12
#bit   tris_db7    = 0x86.7 //rb7  dir db7 pino 13

#bit   oerr        = 0x18.1 // flag de erro de recepo da usart (somente leitura)
#bit   cren        = 0x18.4 //flag cren (usado para habilitar/reset da usart)
#bit   rcif        = 0xc.5  //flag de recepo usart (somente leitura)

#define BUFFER_SIZE 33
#define VALOR_MIN  7
#define TEMPO_SEM_RX_SERIAL  200000
//variveis globais
int8 cnt=0;
char buffer[BUFFER_SIZE];
int1 rec_ok=0,flag_err=0;
int32 time_out=0;
//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 delay_us(10);//
 en=0;

 }

//envia nible para lcd
 void env_nib(int8 b){

 if(b&0x10)db4=1; else db4=0;
 if(b&0x20)db5=1; else db5=0;
 if(b&0x40)db6=1; else db6=0;
 if(b&0x80)db7=1; else db7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(int8 val){
 rs=0;
 env_nib(val);
 env_nib(swap(val));
 delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(int8 val){
 rs=1;
 env_nib(val);
 env_nib(swap(val));
 }

//inicializao do lcd
void lcd_init_(void){
 int8 a;

  rs=0;
  tris_en=0;//direo dos pinos como saida
  tris_rs=0;
  tris_db4=0;
  tris_db5=0;
  tris_db6=0;
  tris_db7=0;
  db4=0;//situao inicial resetados
  db5=0;
  db6=0;
  db7=0;
  en=0;
  delay_ms(50);//delay inicial
  db4=1; db5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); delay_ms(5);}// envia 3,3,3
  db4=0;//envia 2
  lcd_clk();//pulso em 'en'
  delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);

}
//-------------------------FIM DAS ROTINAS DE LCD--------------------------------

//Rotina de interrupo da Usart
//Nesta rotina capturamos a informao de latitude e longitude em um buffer
//usando o dado nomeado de  $GPGLL.

#int_rda
void usart_isr(){
char a;
  a=getc();


  cren=0;//reseta serial para apagar algum erro na recepo
  cren=1;//religa
  if(rec_ok)return; //temos dados ainda sem processar, ento aguarda processamento


  if(cnt<VALOR_MIN){//identifica a sequncia de caracteres desejada: $GPGLL

    if(a=='$' && cnt==0){cnt++; return;}
    else if(a=='G' && cnt==1){cnt++; return;}
    else if(a=='P' && cnt==2){cnt++; return;}
    else if(a=='G' && cnt==3){cnt++; return;}
    else if(a=='L' && cnt==4){cnt++; return;}
    else if(a=='L' && cnt==5){cnt++; return;}
    else if(a==',' && cnt==6){cnt++; return;}
    else {cnt=0; return;}//se sequncia de caracteres for incorreta zera contador
  }



   if(cnt==18){//verifica a presena do caracter 'S'ou 'N' para validao
   if(a=='S'|| a=='N')flag_err=0; else {flag_err=1;rec_ok=1;}
   }


  if((cnt<=BUFFER_SIZE) && (cnt>=VALOR_MIN)){ buffer[cnt++]=a; }//captura dados
  if(cnt>BUFFER_SIZE) {rec_ok=1;}//notifica fim da captura da informao



}

//=============================================================================
 //    ROTINA DE MAIN
 //=============================================================================


void main() {

   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   trisb = 0b00000110;//Define PORTb como saidas
   portb=0;//reseta portb
   lcd_init_(); //Inicializa o display
   env_lcd_c(0x80);//linha 1
   printf(env_lcd_d("  GPS com PIC   "));// nome do aparelho
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d(" Blog Picsource "));//blog
   delay_ms(2000);//tempo de apresentao
   env_lcd_c(1);//limpa tela
   printf(env_lcd_d("Aguarde ..."));//blog
   enable_interrupts(int_rda);//ativa interrupo da serial
   enable_interrupts(global);//ativa interrupo global
   cnt=0;//zera contador

 //=============================================================================
 //   LAO PRINCIPAL DE REPETIO
 //=============================================================================

   for(;;){
    char b;
#define SIZE_LINHA1  13

    if(rec_ok){
      time_out=0;//reseta time_out
      disable_interrupts(int_rda);//desliga recepo da serial
      env_lcd_c(1);//limpa tela e inicia na linha 1



      if(flag_err==0){
        printf(env_lcd_d,"LA=");//latitude
        for(b=VALOR_MIN;b<BUFFER_SIZE;b++){
           if(b==SIZE_LINHA1+VALOR_MIN){
           env_lcd_c(0xc0);//muda para a linha 2
           printf(env_lcd_d,"LO=");//longitude
           }
           printf(env_lcd_d(buffer[b]));//imprime valores capturados no buffer
        }
      }

      else{
//         printf(env_lcd_d,"Aguarde Envio de");
//         env_lcd_c(0xc0); //linha 2
//         printf(env_lcd_d,"Dados Corretos");
          
         
       printf(env_lcd_d,"Espere localizar");
       env_lcd_c(0xc0); //linha 2
       printf(env_lcd_d,"os Satelites... ");
       
        
      }



      cnt=0;//zera contador
      rec_ok=0;//avisa para comear a atualizar com novos dados
      enable_interrupts(int_rda);//religa interrupo da serial

    }//if(rec_ok)
    else{
       time_out++;
       if(time_out>TEMPO_SEM_RX_SERIAL){
         time_out=0;
         env_lcd_c(1);//limpa tela
         printf(env_lcd_d,"Sem Comunicacao");
         env_lcd_c(0xc0);
         printf(env_lcd_d,"Com modulo GPS");
       }
    }

   }//for(;;)

}// main()


//======================Fim do Programa=========================================

// Memory usage: ROM=25%   RAM=30% - 32%









